"{ Package: 'jv:dragonfly/asm' }"

"{ NameSpace: Smalltalk }"

AJInstruction subclass:#AJData
	instanceVariableNames:'data alignment'
	classVariableNames:''
	poolDictionaries:'AJConstants'
	category:'AsmJit-Instructions'
!

AJData comment:'I represent a pure data section in an assembly instruction stream.

Example:
	asm := AJx64Assembler noStackFrame.
	
	"add a raw byte"
	asm db: 16rFF.
	
	"add a raw word"
	asm dw: #[16r34 16r12].
	
	"add a raw double"
	asm dw: #[16r78 16r56 16r34 16r12].
	
	"add a arbitrary sized data section with a byteArray"
	asm data: #[1 2 3 4 5 6 7 8 9 10 11 12 ].'
!

!AJData class methodsFor:'instance creation'!

byte: aByteValue
    ^ self data: (ByteArray with: aByteValue)
!

data: aDataByteArray
    ^ self new
        data: aDataByteArray;
        yourself
!

label: aLabel data: aDataByteArray
    ^ self new
        label: aLabel;
        data: aDataByteArray;
        yourself
! !

!AJData methodsFor:'accessing'!

data
    ^ machineCode
!

data: aByteArray
    "the will be put in the executable."
    machineCode := aByteArray
!

name
    name ifNotNil: [ ^ name ].
    
    "standard data sections"
    self is8 ifTrue: [ ^ 'db' ].
    self is16 ifTrue: [ ^ 'dw' ].
    self is32 ifTrue: [ ^ 'dd' ].
!

size
    ^ self data size
! !

!AJData methodsFor:'emitting code'!

emitCode: asm
    machineCode ifNil: [ machineCode := #[] ]
! !

!AJData methodsFor:'testing'!

is16
    ^ self size = 2
!

is32
    ^ self size = 4
!

is64
    ^ self size = 8
!

is8
    ^ self size = 1
! !

!AJData methodsFor:'visitor'!

accept: anObject
    anObject instructionData: self
! !

